@@ -10,7 +10,7 @@
 // Copyright (C) 2011 mod_csp project
 // License : Apache Licence 2.0
 //
-// version : 0.01
+// version : 0.0.7
 //
 // Last modification by kumatiki
 //
@@ -64,12 +64,21 @@ module AP_MODULE_DECLARE_DATA csp_module;
 #define	CSPFALSE			0
 
 
+struct _param {
+	request_rec * r;			//	request_rec
+	apr_table_t * g;			//	get
+	apr_table_t * p;			//	post
+	apr_table_t * c;			//	cookie
+	int * session_index;
+	struct _sessionTable * session;
+	int timeout;
+	int (* func)();
+};
 
 ////////////////////////////////////////////////////////////////////////////////////////////////
 //		include
 ////////////////////////////////////////////////////////////////////////////////////////////////
 
-
 # 1 "growmem.c" 1
 # 9 "growmem.c"
 typedef struct _growMem {
@@ -155,24 +164,54 @@ static char * fileRead(apr_pool_t * p,char * filename,int * s)
 # 1 "param.c" 1
 # 10 "param.c"
 struct csp_config {
- char * message;
-} csp_config;
+ char * dir;
+ char * iparam;
+ char * lparam;
+ int timeout;
+};
+
+
 
 static void * create_per_dir_config (apr_pool_t *pool, char *arg)
 {
- void * buf = apr_pcalloc(pool, sizeof(csp_config));
+ void * buf = apr_pcalloc(pool, sizeof(struct csp_config));
  struct csp_config *cfg = (struct csp_config*)buf;
- cfg->message = "/tmp/csp";
+ cfg->dir = "/tmp/csp";
+ cfg->lparam="";
+ cfg->iparam="";
+ cfg->timeout = 5;
  return buf;
 }
 
 
 
 
-static const char *cmd_csp_message (cmd_parms *parms, void *mconfig, char *arg){
-  struct csp_config *cfg = (struct csp_config*)mconfig;
-  cfg->message = apr_pstrdup(parms->pool,arg);
-  return 0;
+static const char *cmd_csp_dir (cmd_parms *parms, void *mconfig, char *arg){
+ struct csp_config *cfg = (struct csp_config*)mconfig;
+ cfg->dir = apr_pstrdup(parms->pool,arg);
+ return 0;
+}
+
+static const char *cmd_csp_timeout (cmd_parms *parms, void *mconfig, char *arg){
+ struct csp_config *cfg = (struct csp_config*)mconfig;
+ int tmp = atoi(arg);
+ if(tmp < 5)tmp = 5;
+ if(tmp > 600)tmp = 600;
+ cfg->timeout = tmp;
+ return 0;
+}
+
+
+static const char * cmd_csp_lparam (cmd_parms *parms, void *mconfig, char *arg){
+ struct csp_config *cfg = (struct csp_config*)mconfig;
+ cfg->lparam = apr_pstrdup(parms->pool,arg);
+ return 0;
+}
+
+static const char * cmd_csp_iparam (cmd_parms *parms, void *mconfig, char *arg){
+ struct csp_config *cfg = (struct csp_config*)mconfig;
+ cfg->iparam = apr_pstrdup(parms->pool,arg);
+ return 0;
 }
 
 
@@ -181,14 +220,10 @@ static const char *cmd_csp_message (cmd_parms *parms, void *mconfig, char *arg){
 typedef const char *(*CMD_HAND_TYPE) (cmd_parms*,void*);
 
 static const command_rec csp_cmds[] = {
-  {
- "csp_tmp",
- (CMD_HAND_TYPE)cmd_csp_message,
- 0,
- OR_ALL,
- TAKE1,
- "Error Message"
- },
+ AP_INIT_TAKE1("csp_dir", cmd_csp_dir, NULL, OR_ALL, "set csp module directory"),
+ AP_INIT_TAKE1("csp_timeout", cmd_csp_timeout, NULL, OR_ALL, "set csp timeout"),
+ AP_INIT_RAW_ARGS("csp_lparam", cmd_csp_lparam, NULL, OR_ALL, "set csp default lparam"),
+ AP_INIT_RAW_ARGS("csp_iparam", cmd_csp_iparam, NULL, OR_ALL, "set csp default iparam"),
  {0},
 };
 # 10 "include.c" 2
@@ -584,10 +619,10 @@ static int csptoc(apr_pool_t *p, GROWMEM * ten,char * dstcfile,char * dstdir,GRO
 "	apr_table_t * c;			//	cookie\n"
 "	int * session_index;\n"
 "	struct _sessionTable * session;\n"
+"	int timeout;\n"
 "	int (* func)();				//	test\n"
 "};\n"
 "\n"
-"\n"
 "///////////////////////////////////////////////////////////////////////\n"
 "//\n"
 "///////////////////////////////////////////////////////////////////////\n"
@@ -887,7 +922,7 @@ static int csptoc(apr_pool_t *p, GROWMEM * ten,char * dstcfile,char * dstdir,GRO
 "	signal( SIGHUP , &sigint_handler );\n"
 "	signal( SIGFPE  , &sigint_handler );\n"
 "	signal( SIGALRM  , &sigint_handler );\n"
-"	alarm(5);\n"
+"	alarm(param->timeout);\n"
 "\n"
 "	if( setjmp( jbuf ) == 1 ) {\n"
 "		return 1;\n"
@@ -938,17 +973,6 @@ struct _sessionTable {
  void * key;
 };
 
-struct _param {
- request_rec * r;
- apr_table_t * g;
- apr_table_t * p;
- apr_table_t * c;
- int * session_index;
- struct _sessionTable * session;
- int (* func)();
-};
-
-
 static char * makeSessionID(request_rec *r){
  char * sessionID = apr_psprintf(r->pool,"ID_%lld",apr_time_now());
 }
@@ -1079,8 +1103,15 @@ static int csp_handler(request_rec *r)
     if (r->header_only)
 	    return OK;
 
-//	ap_rprintf(r,"filename = %s<br>\n",r->filename);
-//    return OK;
+	struct csp_config * cfg;
+	cfg = ap_get_module_config(r->per_dir_config, &csp_module);
+
+//	ap_rprintf(r,"csp_dir = %s<br>\n",cfg->dir);
+//	ap_rprintf(r,"csp_timeout = %d<br>\n",cfg->timeout);
+//	ap_rprintf(r,"csp_lparam = %s<br>\n",cfg->lparam);
+//	ap_rprintf(r,"csp_iparam = %s<br>\n",cfg->iparam);
+//	return OK;
+
 
 // 		Error buffer
 //    GROWMEM * errbuffer = getGrowMem(p,1024);
@@ -1115,7 +1146,7 @@ static int csp_handler(request_rec *r)
 		}
 	}
 
-	char * tmpdir = "/tmp/csp";
+	char * tmpdir = cfg->dir;
 	//to copies etc
 	char * dstdir = cat(tmpdir,"/lib",folder,"/");
 	char * dstfile = cat(tmpdir,"/lib",folder,"/",filename);
@@ -1147,12 +1178,14 @@ static int csp_handler(request_rec *r)
 	mtime2 = finfo2.ctime;
 	apr_file_close(fpr);
 
-
 	//It copies it when there is an update. 
 	if(mtime2>mtime1){
-
 		//make folder 
-		apr_dir_make_recursive(dstdir,APR_OS_DEFAULT,p);
+		apr_status_t admrans = apr_dir_make_recursive(dstdir,APR_OS_DEFAULT,p);
+		if(admrans!=APR_SUCCESS){
+			ap_rprintf(r,"csp_dir error : couldn't write");
+			return   OK;
+		}
 
 	   //trans (include develops)
 	    GROWMEM * incl = getGrowMem(p,1024);
@@ -1170,17 +1203,17 @@ static int csp_handler(request_rec *r)
 
 		//compiles
 		chdir(folder);
-		int ccc = system(cat("apxs -Wc,-std=c99 ","-I ",folder," ",gm_getmem(binclude)," -c ",dstcfile," 2>",dstfile,".log"));
+		int ccc = system(cat("apxs -Wc,-std=c99 ","-I ",folder," ",gm_getmem(binclude)," ",cfg->iparam," -c ",dstcfile," 2>",dstfile,".log"));
 		if(ccc!=0){
   			char * errorlog = (char * )apr_pcalloc(p,1024*10);
   			int len = 1024*10;
 		    apr_file_open(&fpr, cat(dstfile,".log"), APR_READ, APR_OS_DEFAULT , p);
 			apr_file_read(fpr,errorlog,&len);
 			apr_file_close(fpr);
-			errordispay(r,"compile error",errorlog);
+			errordispay(r,"compile error : check iparam or csp_iparam",errorlog);
 			return   OK;
 		}
-		system(cat("gcc -shared -Wl,-soname,lib",filename,".so -o ",dstdir,"lib",filename,".so  ",dstofile," ",gm_getmem(blib)," 2>",dstfile,"1.log"));
+		ccc = system(cat("gcc -shared -Wl,-soname,lib",filename,".so -o ",dstdir,"lib",filename,".so  ",dstofile," ",gm_getmem(blib)," ",cfg->lparam," 2>",dstfile,"1.log"));
 		if(ccc!=0){
   			char * errorlog = (char * )apr_pcalloc(p,1024*10);
   			int len = 1024*10;
@@ -1201,7 +1234,7 @@ static int csp_handler(request_rec *r)
 
 	//set parameter 
 	struct _param param  = {
-		r,get,post,cookie,&session_index,session,NULL
+		r,get,post,cookie,&session_index,session,cfg->timeout,NULL
 	};
 
 	int (*fn)(struct _param *);
